# Ninja Ripper 2 .nr files data dump

import os
import sys
import struct

def readString(myfile):
    ''' Reading a null-terminated string byte-by-byte. '''
    str = b''
    while True:
        c = myfile.read(1)
        if not c:
            return None
        if c == b'\0' or c == b'':
            return str.decode()
        else:
            str = str + c

exe_name = "Exe name missing. (renamed or moved rip files)"
if len(os.path.abspath(os.getcwd()).split('_')) >= 3:
    exe_name = os.path.abspath(os.getcwd()).split('_')[-3]
print(exe_name)

dir_list = os.listdir(".")
for f in dir_list:
    if ".nr" in f:
        with open(f, "rb") as work_file:
            print("\n"+"="*40, "\n\nFile:", f,)
            mesh_data = []
            work_file.seek(0, os.SEEK_END)
            file_size = work_file.tell()
            work_file.seek(0, os.SEEK_SET)
            file_magic = work_file.read(4)
            if file_magic != b'NRIP':
                sys.exit("\n\nERROR: Wrong magic bytes!\n")
            file_version = struct.unpack('<I', work_file.read(4))[0]
            if file_version != 1:
                print("\n\nWarning: Not tested .nr version!\n")
            work_file.read(8)
            block_start = work_file.tell()
            while True:
                work_file.seek(block_start)
                if block_start >= file_size:
                    break
                block_size = struct.unpack('<I', work_file.read(4))[0]
                block_name = work_file.read(4)
                if block_name == b'VATR':
                    work_file.read(4)
                    vatr_block_data_count = struct.unpack('<I', work_file.read(4))[0]
                    mesh_data.append(str(vatr_block_data_count))
                    print("\nVertex attributes:", vatr_block_data_count, "(Index, Type, Offset, Size)")
                    for data in range(vatr_block_data_count):
                        data_name = readString(work_file)
                        int1 = struct.unpack('<I', work_file.read(4))[0]
                        int2 = struct.unpack('<I', work_file.read(4))[0]
                        int3 = struct.unpack('<I', work_file.read(4))[0]
                        int4 = struct.unpack('<I', work_file.read(4))[0]
                        print('{:3} {:11} {:3} {:3} {:3} {:3}'.format(data, data_name, int1, int2, int3, int4))
                if block_name == b'INDX':
                    work_file.read(4)
                    indx_block_data_count = struct.unpack('<I', work_file.read(4))[0]
                    indx_block_data_size = struct.unpack('<I', work_file.read(4))[0]
                    print("\nIndices count:", indx_block_data_count)
                if block_name == b'TXTR':
                    work_file.read(4)
                    txtr_block_data_count = struct.unpack('<I', work_file.read(4))[0]
                    mesh_data.append(str(txtr_block_data_count))
                    print("\nTextures count:", txtr_block_data_count)
                    for data in range(txtr_block_data_count):
                        int1 = struct.unpack('<I', work_file.read(4))[0]
                        int2 = struct.unpack('<I', work_file.read(4))[0]
                        data_name = readString(work_file)
                        print(data+1, '{:11} {:3} {:3}'.format(data_name, int1, int2))
                if block_name == b'SHAD':
                    work_file.read(4)
                    shad_block_data_count = struct.unpack('<I', work_file.read(4))[0]
                    mesh_data.append(str(shad_block_data_count))
                    print("\nShaders count:", shad_block_data_count)
                    for data in range(shad_block_data_count):
                        data_name = readString(work_file)
                        print(data+1, data_name)
                if block_name == b'VERT':
                    work_file.read(4)
                    vert_block_data_count = struct.unpack('<I', work_file.read(4))[0]
                    vert_block_data_size = struct.unpack('<I', work_file.read(4))[0]
                    floats_count = vert_block_data_size//4
                    mesh_data.append(str(vert_block_data_size))
                    print("\nVertices count:", vert_block_data_count, "size:", vert_block_data_size, "bytes,", floats_count, "values.")
                    for data in range(vert_block_data_count):
                        vertex_data = []
                        for float_values in range(floats_count):
                            float = struct.unpack('f', work_file.read(4))[0]
                            vertex_data.append('{:10.5f}'.format(float))
                        vertex_string = ' '.join(vertex_data)
                        if data % 100 == 0:
                            print("{:7}".format(data), "vertex:", vertex_string)
                if block_name == b'GEOM':
                    work_file.read(4)
                    some_int = struct.unpack('<I', work_file.read(4))[0] # Total data count?
                    geom_block_data_count = struct.unpack('<I', work_file.read(4))[0]
                    print("\nTotal blocks:", geom_block_data_count, "Counter:", some_int)
                block_start = block_start + block_size
            if exe_name != "Exe name missing. (renamed or moved rip files)":
                mesh_id = exe_name + "-" + '-'.join(mesh_data)
                print("\n\nMesh ID:", mesh_id)
